package net.comp_lot.lib.imagelib;

import java.awt.AlphaComposite;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JPanel;

import net.comp_lot.lib.imagelib.PanelInfo;

/**
 * 画像の描画を管理するための抽象クラス
 * ImageDataの親
 * @author pochi
 */
public abstract class DrawableData {
	private static final AlphaComposite NO_ALPHA = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1);
	
	private float transparent = 1f;
	private boolean visible = true;
	
	/**
	 * 透明度を設定。0~1で0が透明。
	 * @param transparent 透明度
	 */
	public void setTransparent(float transparent) {
		this.transparent = transparent;
	}
	
	float getTransparent() {
		return transparent;
	}
	
	/**
	 * 表示、非表示を設定する。
	 * @param visible 可視性
	 */
	public void setVisible(boolean visible) {
		this.visible = visible;
	}
	
	/**
	 * 表示、非表示の設定を得る。
	 * @return 可視性
	 */
	public boolean getVisible() {
		return visible;
	}
	
	/**
	 * draw()を呼ぶ
	 * @param g 描画先
	 * @param info 描画用の情報
	 * @param panel 呼び出し元
	 */
	public final  void callDraw(Graphics g, PanelInfo info, JPanel panel) {
		if (!visible)
			return;
		Graphics2D g2 = (Graphics2D) g;
		boolean transparent = getTransparent() != 1;
		if (transparent)
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getTransparent()));
		draw(g, info, panel);
		if (transparent)
			g2.setComposite(NO_ALPHA);
	}
	
	/**
	 * 描画
	 * @param g 描画先
	 * @param info 描画用の情報
	 * @param panel 呼び出し元
	 */
	protected abstract void draw(Graphics g, PanelInfo info, JPanel panel);
	
}
