package net.comp_lot.lib.imagelib;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.image.BufferedImage;
import javax.swing.JPanel;

/**
 * 表示する画像の情報のクラス
 * @author pochi
 */
public class ImageData extends AbsoluteDrawableData{
	private BufferedImage image;
	private Image buff;
	private boolean smoothScale = false;
	
	/**
	 * デフォルトコンストラクタ
	 */
	public ImageData() {
		setPlaceAbsolute(0,0);
		setSizeAbsolute(16,16);
	}
	
	/**
	 * 画像を指定するコンストラクタ。
	 * @param image 表示したい画像
	 */
	public ImageData(BufferedImage image) {
		this();
		setImage(image);
	}
	
	/**
	 * 画像を指定する。
	 * @param image 表示したい画像
	 */
	public void setImage(BufferedImage image) {
		this.image = image;
		setSmoothScale(smoothScale);
	}
	
	/**
	 * 幅を保ったまま高さを調整して縦横比を画像に合わせる
	 */
	public void setSizeByWidth() {
		height = (int) (((float)width) / image.getWidth() * image.getHeight());
	}
	
	/**
	 * 幅を保ったまま高さを調整して縦横比を画像に合わせる
	 */
	public void setSizeByHeight() {
		width = (int) (((float)height) / image.getHeight() * image.getWidth());
	}
	
	/**
	 * 縦横比を保ったまま画像を最大化
	 */
	public void setSizeMaximize() {
		float xRatio = ((float)width) / image.getWidth();
		float yRatio = ((float)height) / image.getHeight();
		if (xRatio < yRatio) {
			height = (int) (xRatio * image.getHeight());
		} else {
			width = (int) (yRatio * image.getWidth());
		}
	}
	
	/**
	 * リサイズを滑らかにする。
	 * おそらくメモリを食う。
	 * @param smooth 有効化
	 */
	public void setSmoothScale(boolean smooth) {
		smoothScale = smooth;
		if (smoothScale) {
			buff = image;
		} else {
			buff = null;
		}
	}
	
	/**
	 * @return 画像の幅
	 */
	public int getImageWidth() {
		return image.getWidth();
	}
	/**
	 * @return 画像の高さ
	 */
	public int getImageHeight() {
		return image.getHeight();
	}
	
	@Override
	protected void draw(Graphics g, PanelInfo info, JPanel panel) {
		if (image == null)
			return;
		
		// 場所の決定
//		float x, y;
//		if (startX >= 0) {
//			if (endX >= 0) {
//				x = (startX + endX - widthPercent) / 2f;
//			} else {
//				x = startX;
//			}
//		} else if (endX >= 0){
//			x = endX - widthPercent;
//		} else {
//			x = 0.5f - widthPercent / 2f;
//		}
//		if (startY >= 0) {
//			if (endY >= 0) {
//				y = (startY + endY - heightPercent) / 2f;
//			} else {
//				y = startY;
//			}
//		} else if (endY >= 0){
//			y = endY - heightPercent;
//		} else {
//			y = 0.5f - heightPercent / 2f;
//		}
		
		if (smoothScale) {
			if (image.getWidth() != width || image.getHeight() != height) {
				if (buff.getWidth(panel) != width || buff.getHeight(panel) != height) {
					buff = image.getScaledInstance(width, height, Image.SCALE_SMOOTH);
					MediaTracker mt = new MediaTracker(panel);
					mt.addImage(buff, 1);
					try {
						mt.waitForAll();
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
			}
			g.drawImage(buff, x+info.getCorrectX(), y+info.getCorrectY(),
					width, height, panel);
		} else {
			g.drawImage(image, x+info.getCorrectX(), y+info.getCorrectY(),
					width, height, panel);
		}
	}
}
