package net.comp_lot.lib.imagelib;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.imageio.ImageIO;

/**
 * 画像データを読み込むためのクラス
 * @author pochi
 */
public class ImageLoader {
	private static ImageLoader object = new ImageLoader();
	
	private ImageLoader() {}

	/**
	 * リソースからの画像の読み込み
	 * @param path 画像のパス
	 * @return 画像
	 */
	public static BufferedImage loadResource(String path) {
		InputStream input = object.getClass().getClassLoader().getResourceAsStream(path);
		try {
			return ImageIO.read(input);
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}
	/**
	 * 外部の画像の読み込み
	 * @param path 画像のパス
	 * @return 画像
	 */
	public static BufferedImage loadImage(String path) {
		try {
			return ImageIO.read(new File(path));
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}
}
