package net.comp_lot.lib.imagelib;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import javax.swing.JPanel;

class ImagePanel extends JPanel{
	private static final long serialVersionUID = 4635056583188263142L;
	
	private static final Color CLEAR_COLOR = new Color(240, 240, 240);
	
	private Boolean dataLock = false;
	private List<DrawableData> list = new ArrayList<DrawableData>();
	PanelInfo info = new PanelInfo();
	
	private BufferedImage image = new BufferedImage(1, 1, BufferedImage.TYPE_INT_RGB);
	
	@Override
	protected void paintComponent(Graphics g){
		super.paintComponent(g);
		draw(g);
	}
	
	void forcePaint() {
		if (dataLock)
			return;
		// sizeを求める
		int width = getWidth();//横幅
		int height = getHeight();//高さ
		//Image image = createImage(width, height);
		if (image.getWidth() != width || image.getHeight() != height) {
			image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
			image.setAccelerationPriority(0.8f);
		}
		if (image == null) {
			System.out.println("cannot create image");
			return;
		}
		Graphics2D g2 = (Graphics2D)image.getGraphics();
		g2.setBackground(CLEAR_COLOR);
		g2.setColor(Color.BLACK);
		g2.clearRect(0, 0, width, height);
		draw(g2);
		Graphics g = getGraphics();
		g.drawImage(image, 0, 0, this);
		g.dispose();
	}
	
	
	private void draw(Graphics g) {
		waitLock();
		((Graphics2D) g).setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		info.set(getWidth(), getHeight());
		for (DrawableData data : list) {
			data.callDraw(g, info, this);
		}
		dataLock = false;
	}
	
	List<DrawableData> getImageDataList() {
		return list;
	}
	
	void startDataUpdate() {
		waitLock();
	}
	
	void endDataUpdate() {
		dataLock = false;
	}
	
	/**
	 * アスペクト比を設定する。
	 * 以後、表示領域はその内部として扱われる
	 * @param width 横
	 * @param height 縦
	 */
	void setAspectRatio(int width, int height) {
		info.aspectRatio = 1f * width / height;
	}
	
	void resetAspectRatio() {
		info.aspectRatio = -1;
	}
	
	private void waitLock() {
		synchronized(dataLock) {
			while (dataLock) {
				try {
					TimeUnit.MILLISECONDS.sleep(1);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			dataLock = true;
		}
	}
}
