package net.comp_lot.lib.imagelib;

import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;

import javax.swing.JPanel;

/**
 * データをリスト表示するためのクラス
 * @author pochi
 */
public class ListDrawableData extends AbsoluteDrawableData {
	private List<AbsoluteDrawableData> list = new ArrayList<>();
	private int margin;
	
	/**
	 * @param data リストに追加するデータ
	 */
	public void add(AbsoluteDrawableData data) {
		list.add(data);
	}
	
	/**
	 * リストから削除
	 * @param index 削除する番号
	 * @return 削除したデータ
	 */
	public AbsoluteDrawableData removeData(int index) {
		if (index < 0 || index >= list.size())
			return null;
		return list.remove(index);
	}
	
	/**
	 * リストの中身を全消去
	 */
	public void clear() {
		list.clear();
	}
	
	/**
	 * リストのすべての要素に対して処理を行う
	 * @param consumer 処理
	 */
	public void forEach(Consumer<AbsoluteDrawableData> consumer) {
		for (AbsoluteDrawableData data : list)
			consumer.accept(data);
	}
	
	/**
	 * 中身を取得
	 * @param index 番号
	 * @return 番号に対応した中身
	 */
	public AbsoluteDrawableData get(int index) {
		return list.get(index);
	}
	
	/**
	 * 余白の大きさを設定する
	 * @param margin 余白の大きさ
	 */
	public void setMargin(int margin) {
		this.margin = margin;
	}
	
	/**
	 * クリックされた要素を調べる
	 * @param x x座標
	 * @param y y座標
	 * @return 無ければ-1
	 */
	public int getClicked(int x, int y) {
		for (int i=0;i<list.size();i++) {
			if (list.get(i).contains(x, y))
				return i;
		}
		return -1;
	}

	@Override
	protected void draw(Graphics g, PanelInfo info, JPanel panel) {
		int useHeight = 0;
		int limitHeight = height;
		for (AbsoluteDrawableData data : list) {
			if (useHeight + data.height > limitHeight)
				break;
			if (useHeight >= 0) {
				data.setPlaceAbsolute(x, y + useHeight);
				data.callDraw(g, info, panel);
			}
			useHeight += data.getHeight() + margin;
		}
	}

}
