package net.comp_lot.lib.imagelib;

import java.awt.image.BufferedImage;

/**
 * 割合指定できる画像のクラス
 * @author pochi
 */
public class RatioImageData extends RatioDataAdapter<ImageData> {
	/**
	 * 表示する大きさを個別に指定する
	 */
	public static final int SIZE_BY_BOTH = 0;
	/**
	 * 表示する大きさを幅で決める（画像の縦横比を保つ）
	 */
	public static final int SIZE_BY_WIDTH = 1;
	/**
	 * 表示する大きさを高さで決める（画像の縦横比を保つ）
	 */
	public static final int SIZE_BY_HEIGHT = 2;
	/**
	 * 縦横比を保ちながらできる限り大きく（デフォルト）
	 */
	public static final int SIZE_MAXIMIZE = 3;
	/**
	 * 画像の元の大きさで表示
	 */
	public static final int SIZE_NOT_CHANGE = 4;
	
	private int sizeType = SIZE_MAXIMIZE;
	
	/**
	 * @param image 画像
	 */
	public RatioImageData(BufferedImage image) {
		super(new ImageData(image));
	}
	
	/**
	 * 表示する大きさを設定する。
	 * 片方マイナスの場合はプラスのほうに合わせた倍率（縦横比を保つ）
	 * 両方マイナスの場合は縦横比を保ちながらできる限り大きく
	 * @param width 幅
	 * @param height 高さ
	 */
	@Override
	public void setSize(float width, float height) {
		super.setSize(width, height);
		if (width >= 0) {
			if (height >= 0) {
				sizeType = SIZE_BY_BOTH;
			} else {
				sizeType = SIZE_BY_WIDTH;
			}
		} else {
			if (height >= 0) {
				sizeType = SIZE_BY_HEIGHT;
			} else {
				sizeType = SIZE_MAXIMIZE;
			}
		}
	}
	
	/**
	 * 表示する大きさを設定する。
	 * SIZE_MAXIMIZE または SIZE_NOT_CHANGE
	 * @param type 大きさの決定方法
	 */
	public void setSizeType(int type) {
		switch (type) {
		case SIZE_MAXIMIZE:
			sizeType = SIZE_MAXIMIZE;
			break;
		case SIZE_NOT_CHANGE:
			sizeType = SIZE_NOT_CHANGE;
			break;
		}
	}
	
	@Override
	protected void updateSize() {
		// 大きさの決定
		switch(sizeType) {
		case SIZE_BY_BOTH:
			break;
		case SIZE_BY_WIDTH:
			get().setSizeByWidth();
			break;
		case SIZE_BY_HEIGHT:
			get().setSizeByHeight();
			break;
		case SIZE_MAXIMIZE:
			get().setSizeMaximize();
			break;
		}
	}

}
